import { generateShades } from ".";

const exampleColors = {
  main: "black",
  "secondary-accent": "#fff",
};

test("color shades are generated in a nested structure", () => {
  const colors = generateShades(exampleColors, {
    structure: "nested",
    format: "css-variable",
  });

  expect(colors).toMatchObject({
    main: {
      "10": "rgba(0,0,0,0.1)",
    },
  });
});

test("color shades are generated in a flat structure", () => {
  const colors = generateShades(exampleColors, {
    structure: "flat",
    format: "css-variable",
  });

  expect(colors).toMatchObject({
    "main-10": "rgba(0,0,0,0.1)",
  });
});

test("color shades are generated in a flat structure with a prefix", () => {
  const colors = generateShades(exampleColors, {
    structure: "flat",
    keyPrefix: "color-",
    format: "css-variable",
  });

  expect(colors).toMatchObject({
    "color-main-10": "rgba(0,0,0,0.1)",
  });
});

test("a filtered list of color shades is generated in a flat structure", () => {
  const colors = generateShades(exampleColors, {
    structure: "flat",
    keys: ["main"],
    format: "css-variable",
  });

  expect(colors).toMatchObject({
    "main-10": "rgba(0,0,0,0.1)",
  });
  expect(colors["secondary-accent"]).toBeUndefined();
});
