import * as Colors from "./Colors";
import { generateShades, ColorsInput, ShadesOptions } from "./Colors";

export interface ThemeInput {
  name: string;
  colors: ColorsInput;
  selector?: string;
}

const generateThemeOverride = (input: {
  properties: any;
  selector: string;
}) => {
  const propKeys = Object.keys(input.properties);

  if (propKeys.length > 0) {
    return ` 
    ${input.selector} {${propKeys
      .map((key) => `\t--${key}: ${input.properties[key]};`)
      .join("\n")}}`;
  } else return "";
};

const generateThemeRoot = (input: { properties: any }) => {
  const propKeys = Object.keys(input.properties);

  if (propKeys.length > 0) {
    return `
    :root {${propKeys
      .map((key) => `\t--${key}: ${input.properties[key]};`)
      .join("\n")}}`;
  } else return "";
};

const generateTheme = (options: {
  output?: "CSS";
  themes: ThemeInput[];
  generateColorShades?: boolean | ShadesOptions;
}) => {
  let themes = options.themes;

  if (
    options.generateColorShades !== undefined &&
    options.generateColorShades !== false
  ) {
    themes = themes.map((theme) => {
      const updatedColors = {
        ...generateShades(
          theme.colors,
          typeof options.generateColorShades !== "boolean"
            ? options.generateColorShades
            : undefined
        ),
        ...theme.colors,
      };

      return { ...theme, colors: updatedColors };
    });
  }

  const themeRoot = generateThemeRoot({
    properties: { ...themes[0].colors },
  });

  if (themes.length > 1) {
    const themeOverrides = themes
      .slice(1)
      .map((theme) => {
        const overrideClass = generateThemeOverride({
          properties: { ...theme.colors },
          selector: theme.selector ? theme.selector : "." + theme.name,
        });

        return overrideClass;
      })
      .join("\n");

    return themeRoot + themeOverrides;
  } else return themeRoot;
};

export { generateThemeOverride, generateThemeRoot, generateTheme };
export default {
  Colors,
  generateThemeOverride,
  generateThemeRoot,
  generateTheme,
};
